﻿// VGA 640x480, mit Zeichensatz-Bildschirm 80 Spalten x 30 Zeilen
`default_nettype none

`include "utils/rom.v"
`include "utils/ram.v"

module main(

 input ext_clk,			// 50 MHz externer Takt
 output hsync,
 output vsync,
 output [2:0] R,
 output [2:0] G,
 output [1:0] B
);

wire clk;
wire [7:0] char;
wire [7:0] dat;
wire [9:0] charLin;
reg  [2:0] Pixel;
reg  [9:0] CounterX;	// 10 Bit, range 0..799
reg  [9:0] CounterY;	//  10 Bit, range 0..524
reg  [11:0] Basis;  // Adr. Zeilenanfang
reg  [6:0]  Offs;	 // Pos. in Zeile 
reg  [11:0] ZPos;	 // effektive Zeichenpos. in BWS	


 // Instantisierung der Sub-Module
 pll myPLL (.inclk0(ext_clk), .c0(clk));	// erzeugt stabile 25 MHz
 rom myRom (.addr_a(charLin), .clk(clk), .q_a(dat)); // Zeichensatz
 ram myRam (.q(char), .read_address(ZPos), .clk(clk)); // Bildspeicher

 wire CounterXmaxed = (CounterX==799);
 wire CounterYmaxed = (CounterY==520);

 always @(posedge clk) begin 
 // Zählen in horizontaler Richtung
  if(CounterXmaxed)
   CounterX <= 0;
  else begin 
   CounterX <= CounterX + 1;
	if (dat & (1 << CounterX[2:0])) Pixel <= {{2{CounterY[0]}},1'b1};
	else Pixel <= {2'b0,CounterY[0]};
  end
 // Zählen in vertikaler Richtung
  if(CounterXmaxed) begin 
   if (CounterYmaxed) CounterY <= 0;
	else CounterY <= CounterY + 1;
  end

  if (CounterX[2:0] == 0) begin
   if (CounterX == 0 && CounterY == 0) Basis <= 0;
   else if (CounterX == 0 && CounterY[3:0]==0) Basis <= Basis + 80;
  end
  Offs <= CounterX[9:3];
  ZPos <= Basis + Offs;
 end

 // die beiden Sync Signale erzeugen	
 assign hsync = (CounterX < 656) || (CounterX> 751);
 assign vsync = (CounterY < 490) || (CounterY > 491);	
	
 assign charLin = {char,CounterY[3:1]};  // Zeiger fuer Zeichensatz
	
 // Filter für video enable ableiten
 wire h_enable = (CounterX > 3 && CounterX < 644);
 wire v_enable = (CounterY < 480);
 wire vid_enable = h_enable && v_enable;
 // Pixel für VGA Ausgang schalten
 assign R = {3{vid_enable}} & Pixel;
 assign G = {3{vid_enable}} & Pixel;
 assign B = 2'b00;

endmodule // main

